from typing import Dict, List, Optional
import logging

from sqlalchemy.exc import IntegrityError, SQLAlchemyError, DatabaseError as SqlAlchemyDatabaseError
from sqlalchemy.orm import joinedload
from utils.errors.database import DatabaseError, DuplicateEntryError, DatabaseConnectionError
from models import Book
from database.manager import DatabaseManager

from .author import get_or_create_author
from .book_category import get_or_create_categories

logger = logging.getLogger(__name__)


def fetch_all_books() -> List[Book]:
    """
    Fetches all books from the database.

    :return: A list of all books in the database.
    :raises DatabaseConnectionError: If the connection to the database is interrupted.
    :raises DatabaseError: If any other error occurs while fetching books.
    """
    with DatabaseManager.get_session() as session:
        try:
            return session.query(Book) \
                          .options(
                            joinedload(Book.author),
                            joinedload(Book.categories)
                          ) \
                          .all()
        except SqlAlchemyDatabaseError as e:
            logger.critical("Connection with database interrupted")
            raise DatabaseConnectionError("Connection with database interrupted") from e
        except SQLAlchemyError as e:
            logger.error(f"An error occurred when fetching all books: {e}")
            raise DatabaseError("An error occurred when fetching all books") from e


def create_book(book: Dict[str, object]) -> None:
    """
    Creates a new book in the database.

    :param book: A dictionary containing the book details (title, description, year_published, ISBN, author, and categories).
    :raises DuplicateEntryError: If a book with the same ISBN already exists in the database.
    :raises DatabaseConnectionError: If the connection to the database is interrupted.
    :raises DatabaseError: If any other error occurs while creating the book.
    """
    create_books([book])


def create_books(books: List[Dict[str, object]]) -> None:
    """
    Creates multiple books in the database.

    :param books: A list of dictionaries, each containing the details of a book.
    :raises DuplicateEntryError: If a book with the same ISBN already exists in the database.
    :raises DatabaseConnectionError: If the connection to the database is interrupted.
    :raises DatabaseError: If any other error occurs while creating the books.
    """
    try:
        with DatabaseManager.get_session() as session:
            for book in books:
                logger.debug(f"Attempting to create a new book: {book['title']}")

                # Check if the book already exists
                existing_book = session.query(Book).filter_by(isbn=book["isbn"]).first()

                if existing_book:
                    logger.warning(f"Book with ISBN {book['isbn']} already exists. Skipping.")
                    continue

                author = get_or_create_author(session, book["author"])
                categories = get_or_create_categories(session, book["categories"])

                # Create the new book
                new_book = Book(
                    title=book["title"],
                    description=book["description"],
                    year_published=book["year_published"],
                    isbn=book["isbn"],
                    author=author,
                    categories=categories
                )
                session.add(new_book)

                session.commit()
                logger.info(f"Book {book['title']} successfully created.")
    except IntegrityError as e:
        logger.warning("Data already exists")
        raise DuplicateEntryError("Data already exists in the database") from e
    except SqlAlchemyDatabaseError as e:
        logger.critical("Connection with database interrupted")
        raise DatabaseConnectionError("Connection with database interrupted") from e
    except SQLAlchemyError as e:
        logger.error(f"An error occurred when creating the book: {e}")
        raise DatabaseError("An error occurred when creating the book") from e


def update_book(book: Dict[str, object]) -> None:
    """
    Updates an existing book in the database. Reuses existing authors and categories if they exist.

    :param book: A dictionary containing the updated book details, including the book ID.
    :raises DatabaseError: If the book is not found in the database.
    :raises DuplicateEntryError: If an attempt is made to update the book with duplicate data.
    :raises DatabaseConnectionError: If the connection to the database is interrupted.
    """
    try:
        with DatabaseManager.get_session() as session:
            logger.debug(f"Updating book {book['title']}")

            # Find the existing book
            existing_book = session.query(Book).get(book["id"])
            if not existing_book:
                logger.warning(f"Book with ID {book['id']} not found")
                raise DatabaseError("Book not found in the database")

            # Get or create the author
            author = get_or_create_author(session, book["author"])

            # Get or create the categories
            categories = get_or_create_categories(session, book["categories"])

            # Update the book details
            existing_book.title = book["title"]
            existing_book.description = book["description"]
            existing_book.year_published = book["year_published"]
            existing_book.isbn = book["isbn"]
            existing_book.author = author
            existing_book.categories = categories

            session.commit()
            logger.info(f"{book['title']} successfully updated.")
    except IntegrityError as e:
        logger.warning("Data already exists")
        raise DuplicateEntryError("Data already exists in the database") from e
    except SqlAlchemyDatabaseError as e:
        logger.critical("Connection with database interrupted")
        raise DatabaseConnectionError("Connection with database interrupted") from e
    except SQLAlchemyError as e:
        logger.error(f"An error occurred when updating the book: {e}")
        raise DatabaseError("An error occurred when updating the book") from e


__all__ = ["create_book", "create_books", "update_book", "fetch_all_books"]
