from typing import Dict, List, Optional
import logging

from models import BookCategory
from database.manager import DatabaseManager

logger = logging.getLogger(__name__)


def get_or_create_categories(session, category_names: List[str]) -> List[BookCategory]:
    """
    Checks if categories exist in the database, creates ones that don't.

    :param session: SQLAlchemy session object.
    :param category_names: List of category names.
    :return: List of BookCategory instances (existing or newly created).
    """
    processed_categories = {}  # Cache for already processed categories
    filtered_categories = []

    for category_name in category_names:
        if category_name in processed_categories:
            filtered_categories.append(processed_categories[category_name])
            continue

        existing_category = session.query(BookCategory).filter_by(name=category_name).one_or_none()

        if existing_category is not None:
            logger.debug(f"Category {category_name} already exists. Reusing.")
            processed_categories[category_name] = existing_category
            filtered_categories.append(existing_category)
        else:
            logger.debug(f"Adding new category: {category_name}")
            new_category = BookCategory(name=category_name)
            session.add(new_category)
            processed_categories[category_name] = new_category
            filtered_categories.append(new_category)

    return filtered_categories